//
// Created by philipp on 22.03.18.
//

#ifndef MODULES_LOGGING_H
#define MODULES_LOGGING_H

#include <nodelet/nodelet.h>
#include <ros/ros.h>

#include <boost/asio.hpp>

#include "car/camDataMsg.h"
#include "car/ccDataMsg.h"
#include "car/environmentDataMsg.h"
#include "car/laneDataMsg.h"
#include "car/logEnabledMsg.h"
#include "car/logStringMsg.h"
#include "car/rcEnabledMsg.h"
#include "car/stmDataMsg.h"
#include "car/ussDataMsg.h"

#include "NetworkingLib/ServiceServer.h"

namespace car
{
class Logging : public nodelet::Nodelet
{
public:
    static constexpr std::size_t PORT{10207};
    static constexpr std::size_t MAX_MESSAGE_SIZE{1024};
    static constexpr std::size_t MAX_BUFFER_SIZE{0x100000}; // 1MB

    void onInit() override;

    Logging(ros::NodeHandle & nh, std::string & name);

    Logging();

    ~Logging();

private:
    struct LoggingService
    {
        using RequestMessage = std::string;
        using ResponseMessage = std::string;
    };

    ros::NodeHandle nh_;
    std::string name_;

    ros::Subscriber camData;
    ros::Subscriber ccData;
    ros::Subscriber environmentData;
    ros::Subscriber laneData;
    ros::Subscriber logEnabled;
    ros::Subscriber logString;
    ros::Subscriber rcEnabled;
    ros::Subscriber stmData;
    ros::Subscriber ussData;

    std::mutex loggingMutex;
    boost::asio::streambuf loggingBuffer;
    std::ostream loggingStream;

    networking::Networking net;
    networking::service::Server<LoggingService>::Ptr loggingServer;

    void logMessage(const std::string & str);

    void camDataCallback(const camDataMsg::ConstPtr & inMsg);
    void ccDataCallback(const ccDataMsg::ConstPtr & inMsg);
    void environmentDataCallback(const environmentDataMsg::ConstPtr & inMsg);
    void laneDataCallback(const laneDataMsg::ConstPtr & inMsg);
    void logEnabledCallback(const logEnabledMsg::ConstPtr & inMsg);
    void logStringCallback(const logStringMsg::ConstPtr & inMsg);
    void rcEnabledCallback(const rcEnabledMsg::ConstPtr & inMsg);
    void stmDataCallback(const stmDataMsg::ConstPtr & inMsg);
    void ussDataCallback(const ussDataMsg::ConstPtr & inMsg);
};

}

#endif //MODULES_LOGGING_H
